import logging
import os

from typing import Any, List, Optional, Set, Type, Union

from clipped.config.parser import ConfigParser
from clipped.config.schema import BaseSchemaModel
from clipped.types import Uri
from clipped.utils.paths import check_dirname_exists

from vents.connections import ConnectionCatalog
from vents.connections.connection import Connection
from vents.exceptions import VentError

_logger = logging.getLogger("vents.config.reader")


class AppConfig(BaseSchemaModel):
    project_name: Optional[str] = "Vents"
    project_url: Optional[Uri] = ""
    project_icon: Optional[Uri] = ""
    env_prefix: Optional[str] = "VENTS"
    context_path: Optional[str] = ""
    logger: Any = _logger
    exception: Type[Exception] = VentError
    config_parser: Type[ConfigParser] = ConfigParser
    catalog: Optional[ConnectionCatalog]

    def __init__(self, **data: Any):
        super().__init__(**data)
        self.catalog = self.catalog or self.load_connections_catalog()

    def get_from_env(self, keys: Union[Set[str], List[str], str]) -> Any:
        """
        Returns a variable from one of the list of keys based on the os.env.
        Args:
            keys: list(str). list of keys to check in the environment

        Returns:
            str | None
        """
        keys = keys or []
        if not isinstance(keys, (list, tuple, set)):
            keys = [keys]
        for key in keys:
            value = os.environ.get(key)
            if value:
                if value.lower() == "true":
                    return True
                if value.lower() == "false":
                    return False
                return value
            # Prepend prefix if set
            if self.env_prefix:
                key = "{}_{}".format(self.env_prefix, key)
                value = os.environ.get(key)
                if value:
                    return value

        return None

    def get_from_path(
        self, context_paths: List[str], keys: Union[Set[str], List[str], str]
    ) -> Optional[Any]:
        """
        Returns a variable from one of the list of keys based on a base path.
        Args:
            context_paths: List[str], base path where to look for keys.
            keys: list(str). list of keys to check in the environment

        Returns:
            str | None
        """
        context_paths = context_paths or []
        context_paths = [
            c for c in context_paths if check_dirname_exists(c, is_dir=True)
        ]
        if not context_paths:
            return None

        keys = keys or []
        if not isinstance(keys, (list, tuple)):
            keys = [keys]  # type: ignore
        for key in keys:
            for context_path in context_paths:
                key_path = os.path.join(context_path, key)
                if not os.path.exists(key_path):
                    continue
                with open(key_path) as f:
                    value = f.read()
                    if value:
                        if value.lower() == "true":
                            return True
                        if value.lower() == "false":
                            return False
                        return value

        return None

    def get_connections_catalog_env_name(self) -> str:
        env_name = "CONNECTIONS_CATALOG"
        if self.env_prefix:
            env_name = "{}_{}".format(self.env_prefix, env_name)
        return env_name

    @staticmethod
    def get_connections_catalog(
        connections: Optional[List[Connection]],
    ) -> Optional[ConnectionCatalog]:
        if not connections:
            return None
        return ConnectionCatalog(connections=connections)

    def set_connections_catalog(self, connections: Optional[List[Connection]]):
        self.catalog = self.get_connections_catalog(connections)

    def load_connections_catalog(self) -> Optional[ConnectionCatalog]:
        catalog_env_name = self.get_connections_catalog_env_name()
        connections_catalog = os.environ.get(catalog_env_name)
        if not connections_catalog:
            return None
        return ConnectionCatalog.read(connections_catalog, config_type=".json")

    def get_connection_for(self, name: Optional[str]) -> Optional[Connection]:
        """Checks if a connection has a mount path exported"""
        if not name or not self.catalog:
            return None

        return self.catalog.connections_by_names.get(name)

    def read_keys(self, context_paths: List[str], keys: List[str]) -> Optional[Any]:
        """Returns a variable by checking first a context path and then in the environment."""
        keys = (
            {k.lower() for k in keys}
            | {k.upper() for k in keys}
            | {"".join(k.lower().split("_")) for k in keys}
        )
        if context_paths:
            value = self.get_from_path(context_paths=context_paths, keys=keys)
            if value is not None:
                return value
        return self.get_from_env(keys)



import os

from typing import TYPE_CHECKING, Optional

from vents.providers.aws.base import (
    get_aws_access_key_id,
    get_aws_secret_access_key,
    get_aws_security_token,
    get_aws_use_ssl,
    get_aws_verify_ssl,
    get_endpoint_url,
    get_region,
)
from vents.providers.base import BaseService

if TYPE_CHECKING:
    from vents.connections.connection import Connection


class AWSService(BaseService):
    resource: Optional[str]
    region: Optional[str]
    endpoint_url: Optional[str]
    access_key_id: Optional[str]
    secret_access_key: Optional[str]
    session_token: Optional[str]
    verify_ssl: Optional[bool]
    use_ssl: Optional[bool]

    @classmethod
    def load_from_connection(
        cls, connection: Optional["Connection"]
    ) -> Optional["AWSService"]:
        # Check if there are mounting based on secrets/configmaps
        context_paths = []
        if connection:
            if connection.secret and connection.secret.mount_path:
                context_paths.append(connection.secret.mount_path)
            if connection.config_map and connection.config_map.mount_path:
                context_paths.append(connection.config_map.mount_path)
        region = get_region(context_paths=context_paths)
        endpoint_url = get_endpoint_url(context_paths=context_paths)
        access_key_id = get_aws_access_key_id(context_paths=context_paths)
        secret_access_key = get_aws_secret_access_key(context_paths=context_paths)
        verify_ssl = get_aws_verify_ssl(context_paths=context_paths)
        use_ssl = get_aws_use_ssl(context_paths=context_paths)
        session_token = get_aws_security_token(context_paths=context_paths)
        return cls(
            region=region,
            endpoint_url=endpoint_url,
            access_key_id=access_key_id,
            secret_access_key=secret_access_key,
            session_token=session_token,
            verify_ssl=verify_ssl,
            use_ssl=use_ssl,
        )

    def _set_session(self):
        import boto3

        self._session = boto3.session.Session(
            aws_access_key_id=self.access_key_id,
            aws_secret_access_key=self.secret_access_key,
            aws_session_token=self.session_token,
            region_name=self.region,
        )

    def set_env_vars(self):
        if self.endpoint_url:
            os.environ["AWS_ENDPOINT_URL"] = self.endpoint_url
        if self.access_key_id:
            os.environ["AWS_ACCESS_KEY_ID"] = self.access_key_id
        if self.secret_access_key:
            os.environ["AWS_SECRET_ACCESS_KEY"] = self.secret_access_key
        if self.session_token:
            os.environ["AWS_SECURITY_TOKEN"] = self.session_token
        if self.region:
            os.environ["AWS_REGION"] = self.region
        if self.use_ssl is not None:
            os.environ["AWS_USE_SSL"] = str(self.use_ssl)
        if self.verify_ssl is not None:
            os.environ["AWS_VERIFY_SSL"] = str(self.verify_ssl)

    def get_client(self):
        return self.session.client(
            service_name=self.resource,
            region_name=self.region,
            endpoint_url=self.endpoint_url,
            use_ssl=self.use_ssl,
            verify=self.verify_ssl,
        )

    def get_resource(self):
        return self.session.resource(
            self.resource,
            endpoint_url=self.endpoint_url,
            use_ssl=self.use_ssl,
            verify=self.verify_ssl,
        )



from datetime import datetime
from typing import Dict, Optional

from clipped.config.schema import BaseSchemaModel
from clipped.types import Uri


class NotificationSpec(BaseSchemaModel):
    title: str
    description: str
    details: str
    fallback: Optional[str]
    context: Optional[Dict]
    url: Optional[Uri]
    color: Optional[str]
    ts: Optional[datetime]
